@@ -12,38 +12,94 @@ from sys import modules
 from types import SimpleNamespace
 from typing import Optional
 
-from .command import Command, Schedule
-from .const import (
+from .const import DISCOVER_ALL, DISCOVER_PARAMS, DISCOVER_SCHEMA, DISCOVER_STATUS
+from .devices import BdrSwitch, Device, DhwSensor
+from .entities import Entity, discover_decorator
+from .protocol import Command, Schedule
+from .protocol.const import (
     _000C_DEVICE,
     ATTR_DEVICES,
     ATTR_DHW_SENSOR,
     ATTR_DHW_VALVE,
     ATTR_DHW_VALVE_HTG,
     ATTR_HEAT_DEMAND,
+    ATTR_NAME,
     ATTR_RELAY_DEMAND,
+    ATTR_RELAY_FAILSAFE,
     ATTR_SETPOINT,
     ATTR_TEMP,
     ATTR_WINDOW_OPEN,
     ATTR_ZONE_SENSOR,
     ATTR_ZONE_TYPE,
     DEVICE_HAS_ZONE_SENSOR,
-    DISCOVER_ALL,
-    DISCOVER_PARAMS,
-    DISCOVER_SCHEMA,
-    DISCOVER_STATUS,
     ZONE_CLASS_MAP,
     ZONE_TYPE_MAP,
     ZONE_TYPE_SLUGS,
     ZoneMode,
-    __dev_mode__,
 )
-from .devices import BdrSwitch, Device, DhwSensor, Entity
-from .exceptions import CorruptStateError
-from .helpers import schedule_task
+from .protocol.exceptions import CorruptStateError
 
 # from .ramses import RAMSES_ZONES, RAMSES_ZONES_ALL
-
-I_, RQ, RP, W_ = " I", "RQ", "RP", " W"
+from .protocol import I_, RP, RQ, W_, __dev_mode__  # noqa: F401, isort: skip
+from .protocol import (  # noqa: F401, isort: skip
+    _0001,
+    _0002,
+    _0004,
+    _0005,
+    _0006,
+    _0008,
+    _0009,
+    _000A,
+    _000C,
+    _000E,
+    _0016,
+    _0100,
+    _01D0,
+    _01E9,
+    _0404,
+    _0418,
+    _042F,
+    _1030,
+    _1060,
+    _1090,
+    _10A0,
+    _10E0,
+    _1100,
+    _1260,
+    _1280,
+    _1290,
+    _1298,
+    _12A0,
+    _12B0,
+    _12C0,
+    _12C8,
+    _1F09,
+    _1F41,
+    _1FC9,
+    _1FD4,
+    _2249,
+    _22C9,
+    _22D0,
+    _22D9,
+    _22F1,
+    _22F3,
+    _2309,
+    _2349,
+    _2D49,
+    _2E04,
+    _30C9,
+    _3120,
+    _313F,
+    _3150,
+    _31D9,
+    _31DA,
+    _31E0,
+    _3220,
+    _3B00,
+    _3EF0,
+    _3EF1,
+    _PUZZ,
+)
 
 DEV_MODE = __dev_mode__ and False
 
@@ -52,7 +108,7 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-ZONE_CLASS = SimpleNamespace(
+ZONE_TYPE = SimpleNamespace(
     DHW="DHW",  # Stored HW (not a zone)
     ELE="ELE",  # Electric
     MIX="MIX",  # Mix valve
@@ -65,7 +121,7 @@ ZONE_CLASS = SimpleNamespace(
 class ZoneBase(Entity):
     """The Zone/DHW base class."""
 
-    # __zon_class__ = None  # NOTE: this would cause problems
+    # _TYPE = None  # NOTE: this would cause problems
 
     def __init__(self, evo, zone_idx) -> None:
         _LOGGER.debug("Creating a Zone: %s_%s (%s)", evo, zone_idx, self.__class__)
@@ -81,15 +137,14 @@ class ZoneBase(Entity):
         return f"{self.id} ({self.heating_type})"
 
     def __lt__(self, other) -> bool:
-        if not hasattr(other, "idx"):
+        if not isinstance(other, ZoneBase):
             return NotImplemented
         return self.idx < other.idx
 
     def _set_system(self, parent, zone_idx):
         """Set the zone's parent system, after validating it."""
 
-        # these imports are here to prevent circular references
-        from .systems import System
+        from .systems import System  # to prevent circular references
 
         try:
             if zone_idx != "HW" and int(zone_idx, 16) >= parent.max_zones:
@@ -100,7 +155,7 @@ class ZoneBase(Entity):
         if not isinstance(parent, System):
             raise TypeError(f"{self}: parent must be a System, not {parent}")
 
-        if zone_idx != "HW":
+        if zone_idx != "HW":  # or: FA?
             if self.idx in parent.zone_by_idx:
                 raise LookupError(f"{self}: duplicate zone_idx: {zone_idx}")
             parent.zone_by_idx[zone_idx] = self
@@ -111,20 +166,9 @@ class ZoneBase(Entity):
         return parent, parent._ctl
 
     def _handle_msg(self, msg) -> None:
-        """Validate packets by verb/code."""
-        # TODO: assert msg.src is self, "Devices should only keep msgs they sent"
+        assert msg.src is self._ctl, f"msg inappropriately routed to {self}"
         super()._handle_msg(msg)
 
-        # if not self._zone_type:
-        #     return
-        # ramses_zones = RAMSES_ZONES.get(self._zone_type, RAMSES_ZONES_ALL)
-        # assert (
-        #     msg.code in ramses_zones
-        # ), f"Unknown code for {str(self)}: {msg.verb}/{msg.code}"
-        # assert (
-        #     msg.verb in ramses_zones[msg.code]
-        # ), f"Unknown verb for {str(self)}: {msg.verb}/{msg.code}"
-
     def _send_cmd(self, code, **kwargs) -> None:
         dest = kwargs.pop("dest_addr", self._ctl.id)
         payload = kwargs.pop("payload", f"{self.idx}00")
@@ -133,7 +177,7 @@ class ZoneBase(Entity):
     @property
     def heating_type(self) -> Optional[str]:
         """Return the type of the zone/DHW (e.g. electric_zone, stored_dhw)."""
-        return self.__zon_class__
+        return self._TYPE
 
 
 class ZoneSchedule:  # 0404  # TODO: add for DHW
@@ -144,21 +188,25 @@ class ZoneSchedule:  # 0404  # TODO: add for DHW
 
         self._schedule = Schedule(self)
 
+    # @discover_decorator
     # def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-
     #     if discover_flag & DISCOVER_STATUS:  # TODO: add back in
     #         self._loop.create_task(self.get_schedule())  # 0404
 
-    def _handle_msg(self, msg) -> bool:
-        super()._handle_msg(msg)
+    #     def _handle_msg(self, msg) -> bool:
+    #     super()._handle_msg(msg)
 
-        if msg.code == "0404" and msg.verb == RP:
-            _LOGGER.debug("Zone(%s): Received RP/0404 (schedule) pkt", self)
+    #     if msg.code == _0404 and msg.verb != RQ:
+    #         _LOGGER.debug("Zone(%s): Received RP/0404 (schedule) pkt", self)
+
+    @property
+    def schedule(self) -> dict:
+        if self._schedule:
+            return self._schedule.schedule.get("schedule")
 
     async def get_schedule(self, force_refresh=None) -> Optional[dict]:
-        schedule = await self._schedule.get_schedule(force_refresh=force_refresh)
-        if schedule:
-            return schedule["schedule"]
+        await self._schedule.get_schedule(force_refresh=force_refresh)
+        return self.schedule
 
     async def set_schedule(self, schedule) -> None:
         schedule = {"zone_idx": self.idx, "schedule": schedule}
@@ -166,32 +214,40 @@ class ZoneSchedule:  # 0404  # TODO: add for DHW
 
     @property
     def status(self) -> dict:
-        return {**super().status, "schedule": self._schedule.schedule.get("schedule")}
+        return {
+            **super().status,
+            "schedule": self.schedule,
+        }
 
 
 class RelayDemand:  # 0008
     """Not all zones call for heat."""
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
         if discover_flag & DISCOVER_STATUS:
-            self._send_cmd("0008")  # , payload=self.idx
+            self._send_cmd(_0008)  # , payload=self.idx
 
     @property
     def relay_demand(self) -> Optional[float]:  # 0008
-        if "0008" in self._msgs:
-            return self._msgs["0008"].payload[ATTR_RELAY_DEMAND]
+        # if _0008 in self._msgs:
+        #     return self._msgs[_0008].payload[ATTR_RELAY_DEMAND]
+        return self._msg_value(_0008, key=ATTR_RELAY_DEMAND)
 
     @property
     def status(self) -> dict:
-        return {**super().status, ATTR_RELAY_DEMAND: self.relay_demand}
+        return {
+            **super().status,
+            ATTR_RELAY_DEMAND: self.relay_demand,
+        }
 
 
-class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
+class DhwZone(ZoneSchedule, ZoneBase):  # CS92A  # TODO: add Schedule
     """The DHW class."""
 
-    __zon_class__ = ZONE_CLASS.DHW
+    _TYPE = ZONE_TYPE.DHW
 
     def __init__(
         self, ctl, zone_idx="HW", sensor=None, dhw_valve=None, htg_valve=None
@@ -199,22 +255,14 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
         super().__init__(ctl, zone_idx)
 
         ctl._set_dhw(self)
-        # if profile == ZONE_CLASS.DHW and evo.dhw is None:
+        # if profile == ZONE_TYPE.DHW and evo.dhw is None:
         #     evo.dhw = zone
 
         self._dhw_sensor = None
         self._dhw_valve = None
         self._htg_valve = None
 
-        self._zone_type = ZONE_CLASS.DHW
-
-        self._dhw_mode = None
-        self._dhw_params = None
-        self._dhw_temp = None
-        self._relay_demand = None
-        self._relay_failsafe = None
-
-        self._heat_demand = None
+        self._zone_type = ZONE_TYPE.DHW
 
         if sensor:
             self._set_sensor(sensor)
@@ -223,6 +271,7 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
         if htg_valve:
             self._set_htg_valve(htg_valve)
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # super()._discover(discover_flag=discover_flag)
 
@@ -231,7 +280,7 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
 
         if discover_flag & DISCOVER_SCHEMA:
             [  # 000C: find the DHW relay(s), if any, see: _000C_DEVICE_TYPE
-                self._send_cmd("000C", payload=dev_type)
+                self._send_cmd(_000C, payload=dev_type)
                 for dev_type in (
                     f"00{_000C_DEVICE.DHW_SENSOR}",
                     f"00{_000C_DEVICE.DHW}",
@@ -240,34 +289,16 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
             ]
 
         if discover_flag & DISCOVER_PARAMS:
-            self._gwy.send_cmd(
-                Command.get_dhw_params(self._ctl.id), period=td(hours=12)
-            )
+            self._gwy.send_cmd(Command.get_dhw_params(self._ctl.id))
 
         if discover_flag & DISCOVER_STATUS:
-            self._gwy.send_cmd(Command.get_dhw_mode(self._ctl.id), period=td(hours=12))
-            self._gwy.send_cmd(
-                Command.get_dhw_temp(self._ctl.id), period=td(minutes=30)
-            )
+            self._gwy.send_cmd(Command.get_dhw_mode(self._ctl.id))
+            self._gwy.send_cmd(Command.get_dhw_temp(self._ctl.id))
 
     def _handle_msg(self, msg) -> bool:
+        # assert msg.src is self._ctl, f"msg inappropriately routed to {self}"
         super()._handle_msg(msg)
 
-        if msg.code == "0008":
-            self._relay_demand = msg
-
-        elif msg.code == "0009":
-            self._relay_failsafe = msg
-
-        elif msg.code == "10A0":
-            self._dhw_params = msg
-
-        elif msg.code == "1260":
-            self._dhw_temp = msg
-
-        elif msg.code == "1F41":
-            self._dhw_mode = msg
-
     def _set_dhw_device(self, new_dev, old_dev, attr_name, dev_class, domain_id):
         if old_dev is new_dev:
             return old_dev
@@ -333,18 +364,15 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
 
     @property
     def config(self) -> Optional[dict]:  # 10A0
-        result = self._msg_payload(self._dhw_params)
-        if result:
-            return {k: v for k, v in result.items() if k != "dhw_idx"}
+        return self._msg_value(_10A0)
 
     @property
     def mode(self) -> Optional[dict]:  # 1F41
-        # active (setpoint), mode, until
-        return self._msg_payload(self._dhw_mode)
+        return self._msg_value(_1F41)
 
     @property
     def setpoint(self) -> Optional[float]:  # 1F41
-        return self._msg_payload(self._dhw_params, ATTR_SETPOINT)
+        return self._msg_value(_1F41, key=ATTR_SETPOINT)
 
     @setpoint.setter
     def setpoint(self, value) -> None:  # 1F41
@@ -352,19 +380,19 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
 
     @property
     def temperature(self) -> Optional[float]:  # 1260
-        return self._msg_payload(self._dhw_temp, ATTR_TEMP)
+        return self._msg_value(_1260, key=ATTR_TEMP)
 
     @property
     def heat_demand(self) -> Optional[float]:  # 3150
-        return self._msg_payload(self._heat_demand, ATTR_HEAT_DEMAND)
+        return self._msg_value(_3150, key=ATTR_HEAT_DEMAND)
 
     @property
     def relay_demand(self) -> Optional[float]:  # 0008
-        return self._msg_payload(self._relay_demand, "relay_demand")
+        return self._msg_value(_0008, key=ATTR_RELAY_DEMAND)
 
     @property  # only seen with FC, but seems should pair with 0008?
     def relay_failsafe(self) -> Optional[float]:  # 0009
-        return self._msg_payload(self._relay_failsafe, "relay_failsafe")
+        return self._msg_value(_0009, key=ATTR_RELAY_FAILSAFE)
 
     def set_mode(self, mode=None, active=None, until=None) -> Task:
         """Set the DHW mode (mode, active, until)."""
@@ -383,13 +411,13 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
 
     def set_config(self, setpoint=None, overrun=None, differential=None) -> Task:
         """Set the DHW parameters (setpoint, overrun, differential)."""
-        # if self._dhw_params:  # 10A0
+        # dhw_params = self._msg_value(_10A0)
         # if setpoint is None:
-        #     setpoint = self._msg_payload(self._dhw_params, ATTR_SETPOINT)
+        #     setpoint = dhw_params[ATTR_SETPOINT]
         # if overrun is None:
-        #     overrun = self._msg_payload(self._dhw_params, "overrun")
+        #     overrun = dhw_params["overrun"]
         # if differential is None:
-        #     setpoint = self._msg_payload(self._dhw_params, "differential")
+        #     setpoint = dhw_params["differential"]
 
         cmd = Command.set_dhw_params(self._ctl.id, setpoint, overrun, differential)
         return self._gwy.send_cmd(cmd)
@@ -421,7 +449,7 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
 class Zone(ZoneSchedule, ZoneBase):
     """The Zone base class."""
 
-    __zon_class__ = None  # Unknown
+    _TYPE = None  # Unknown
 
     def __init__(self, evo, zone_idx, sensor=None, actuators=None) -> None:
         """Create a zone.
@@ -441,85 +469,56 @@ class Zone(ZoneSchedule, ZoneBase):
         self.device_by_id = {}
         self._sensor = None
 
-        self._heat_demand = None  # Not used by ELE
-        self._mode = None
-        # self._name = None  # from super()
-        self._setpoint = None
-        self._temperature = None
-        self._zone_config = None
-
-        # these needed here, as we can't use those __init__()s
-        self._mix_config = None  # MIX
-        self._ufh_setpoint = None  # UFH
-        self._window_open = None  # RAD (or: ALL)
-        self._actuator_state = None  # ELE, ZON
-
         # self._schedule = Schedule(self)  # TODO:
 
         if sensor:
             self._set_sensor(sensor)
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
         # TODO: add code to determine zone type if it doesn't have one, using 0005s
         if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.ALL}")
-            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.ALL_SENSOR}")
+            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.ALL}")
+            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.ALL_SENSOR}")
 
         if discover_flag & DISCOVER_PARAMS:
             self._gwy.send_cmd(Command.get_zone_config(self._ctl.id, self.idx))
-            self._gwy.send_cmd(
-                Command.get_zone_name(self._ctl.id, self.idx), period=td(hours=4)
-            )
+            self._gwy.send_cmd(Command.get_zone_name(self._ctl.id, self.idx))
 
         if discover_flag & DISCOVER_STATUS:  # every 1h, CTL will not respond to a 3150
-            self._gwy.send_cmd(
-                Command.get_zone_mode(self._ctl.id, self.idx), period=td(minutes=30)
-            )
-            self._gwy.send_cmd(Command.get_zone_temperature(self._ctl.id, self.idx))
+            self._gwy.send_cmd(Command.get_zone_mode(self._ctl.id, self.idx))
+            self._gwy.send_cmd(Command.get_zone_temp(self._ctl.id, self.idx))
             self._gwy.send_cmd(Command.get_zone_window_state(self._ctl.id, self.idx))
 
         # start collecting the schedule
         # self._schedule.req_schedule()  # , restart=True) start collecting schedule
 
     def _handle_msg(self, msg) -> bool:
+        assert msg.src is self._ctl and (
+            isinstance(msg.payload, dict)
+            or [d for d in msg.payload if d["zone_idx"] == self.idx]
+        ), f"msg inappropriately routed to {self}"
+
+        assert msg.src is self._ctl and (
+            isinstance(msg.payload, list) or msg.payload["zone_idx"] == self.idx
+        ), f"msg inappropriately routed to {self}"
+
         super()._handle_msg(msg)
 
-        if isinstance(msg.payload, list):
-            assert self.idx in [d["zone_idx"] for d in msg.payload]
-
-        if msg.code == "0004":
-            self._name = msg
-
         # not UFH (it seems), but ELE or VAL; and possibly a MIX support 0008 too
-        elif msg.code in ("0008", "0009"):  # TODO: how to determine is/isn't MIX?
+        if msg.code in (_0008, _0009):  # TODO: how to determine is/isn't MIX?
             assert msg.src.type in ("01", "13"), msg.src.type  # 01 as a stat
             assert self._zone_type in (None, "ELE", "VAL", "MIX"), self._zone_type
 
             if self._zone_type is None:
                 self._set_zone_type("ELE")  # might eventually be: "VAL"
 
-        elif msg.code == "000A":
-            self._zone_config = msg
-
-        elif msg.code == "12B0":
-            self._window_open = msg
-
-        elif msg.code == "2309" and msg.verb in (I_, RP):  # setpoint
-            assert msg.src.type == "01", "coding error zxw"
-            self._setpoint = msg
-
-        elif msg.code == "2349" and msg.verb in (I_, RP):  # mode, setpoint
-            assert msg.src.type == "01", "coding error zxx"
-            self._mode = msg
-            self._setpoint = msg
-
-        elif msg.code == "30C9" and msg.verb in (I_, RP):  # used by sensor matching
+        elif msg.code == _30C9 and msg.verb in (I_, RP):  # used by sensor matching
             assert msg.src.type in DEVICE_HAS_ZONE_SENSOR + ("01",), "coding error"
-            self._temperature = msg
 
-        elif msg.code == "3150":  # TODO: and msg.verb in (I_, RP)?
+        elif msg.code == _3150:  # TODO: and msg.verb in (I_, RP)?
             assert msg.src.type in ("00", "02", "04", "13")
             assert self._zone_type in (None, "RAD", "UFH", "VAL")  # MIX/ELE don't 3150
 
@@ -527,8 +526,8 @@ class Zone(ZoneSchedule, ZoneBase):
                 zone_type = ZONE_CLASS_MAP[msg.src.type]
                 self._set_zone_type("VAL" if zone_type == "ELE" else zone_type)
 
-        # elif "zone_idx" in msg.payload:
-        #     pass
+    def _msg_value(self, *args, **kwargs):
+        return super()._msg_value(*args, **kwargs, zone_idx=self.idx)
 
     @property
     def sensor(self) -> Device:
@@ -571,7 +570,7 @@ class Zone(ZoneSchedule, ZoneBase):
         if "02" in dev_types:
             zone_type = "UFH"
         elif "13" in dev_types:
-            zone_type = "VAL" if "3150" in self._msgs else "ELE"
+            zone_type = "VAL" if _3150 in self._msgs else "ELE"
         # elif "??" in dev_types:  # TODO:
         #     zone_type = "MIX"
         elif "04" in dev_types or "00" in dev_types:
@@ -596,7 +595,7 @@ class Zone(ZoneSchedule, ZoneBase):
         """
 
         _type = ZONE_TYPE_SLUGS.get(zone_type, zone_type)
-        if _type not in ZONE_BY_CLASS_ID:
+        if _type not in ZONE_BY_TYPE:
             raise ValueError(f"Not a known zone_type: {zone_type}")
 
         if self._zone_type == _type:
@@ -609,13 +608,13 @@ class Zone(ZoneSchedule, ZoneBase):
             )
 
         self._zone_type = _type
-        self.__class__ = ZONE_BY_CLASS_ID[_type]
+        self.__class__ = ZONE_BY_TYPE[_type]
         self._discover()  # TODO: needs tidyup (ref #67)
 
     @property
-    def name(self) -> Optional[str]:
+    def name(self) -> Optional[str]:  # 0004
         """Return the name of the zone."""
-        return self._msg_payload(self._name, "name")
+        return self._msg_value(_0004, key=ATTR_NAME)
 
     @name.setter
     def name(self, value) -> Optional[str]:
@@ -624,52 +623,27 @@ class Zone(ZoneSchedule, ZoneBase):
 
         # async def get_name(self, force_refresh=None) -> Optional[str]:
         #     """Return the name of the zone."""
-        #     if not force_refresh and self._name is not None:
-        #         return self._msg_payload(self._name, "name")
+        #     if not force_refresh:
+        #         return self.name
 
         #     self._name = None
-        #     self._send_cmd("0004", payload=f"{self.idx}00")
+        #     self._send_cmd(_0004, payload=f"{self.idx}00")
         #     while self._name is None:
         #         await asyncio.sleep(0.05)
 
-        #     return self._msg_payload(self._name, "name")
+        #     return self.name
 
     @property
     def config(self) -> Optional[dict]:  # 000A
-        if not self._zone_config or self._zone_config.is_expired:
-            return
-
-        elif isinstance(self._zone_config.payload, dict):
-            result = self._msg_payload(self._zone_config)
-
-        elif isinstance(self._zone_config.payload, list):
-            tmp = [z for z in self._zone_config.payload if z["zone_idx"] == self.idx]
-            result = {k: v for k, v in tmp[0].items() if k[:1] != "_"}
-
-        if result:
-            return {k: v for k, v in result.items() if k != "zone_idx"}
+        return self._msg_value(_000A)
 
     @property
     def mode(self) -> Optional[dict]:  # 2349
-        # setpoint, mode, until
-        if not self._mode or self._mode.is_expired:
-            return
-
-        result = self._msg_payload(self._mode)
-        if result:
-            return {k: v for k, v in result.items() if k != "zone_idx"}
+        return self._msg_value(_2349)
 
     @property
     def setpoint(self) -> Optional[float]:  # 2309 (2349 is a superset of 2309)
-        if not self._setpoint or self._setpoint.is_expired:
-            return
-
-        elif isinstance(self._setpoint.payload, dict):
-            return self._msg_payload(self._setpoint, ATTR_SETPOINT)
-
-        elif isinstance(self._setpoint.payload, list):
-            _zone = [z for z in self._setpoint.payload if z["zone_idx"] == self.idx]
-            return _zone[0][ATTR_SETPOINT]
+        return self._msg_value((_2309, _2349), key=ATTR_SETPOINT)
 
     @setpoint.setter
     def setpoint(self, value) -> None:
@@ -684,22 +658,7 @@ class Zone(ZoneSchedule, ZoneBase):
 
     @property
     def temperature(self) -> Optional[float]:  # 30C9
-        # if not self._temperature or self._temperature.is_expired:
-        #       # NOTE: CTL (as sensor) won't have this attr...
-        #     if self.sensor and self.sensor.temperature:
-        #         self._temperature = self.sensor._temp
-
-        if not self._temperature or (
-            self._temperature.is_expired == self._temperature.HAS_EXPIRED
-        ):
-            return
-
-        elif isinstance(self._temperature.payload, dict):
-            return self._msg_payload(self._temperature, ATTR_TEMP)
-
-        elif isinstance(self._temperature.payload, list):
-            tmp = [z for z in self._temperature.payload if z["zone_idx"] == self.idx]
-            return tmp[0][ATTR_TEMP]
+        return self._msg_value(_30C9, key=ATTR_TEMP)
 
     @property
     def heat_demand(self) -> Optional[float]:  # 3150
@@ -769,16 +728,17 @@ class Zone(ZoneSchedule, ZoneBase):
         """Return the zone's schema (type, devices)."""
         if not self._sensor:
             sensor_schema = None
-        elif getattr(self._sensor, "_30C9_faked", None) is None:
+        elif getattr(self._sensor, "_fake_30C9", None) is None:
             # NOTE: CTL (as sensor) won't have this attr...
             sensor_schema = self._sensor.id
         else:
             sensor_schema = {
                 "device_id": self._sensor.id,
-                "is_faked": self._sensor._30C9_faked,
+                "is_faked": self._sensor._fake_30C9,
             }
 
         return {
+            "_name": self.name,
             ATTR_ZONE_TYPE: self.heating_type,
             ATTR_ZONE_SENSOR: sensor_schema,
             ATTR_DEVICES: [d.id for d in self.devices],
@@ -800,24 +760,25 @@ class Zone(ZoneSchedule, ZoneBase):
 class EleZone(RelayDemand, Zone):  # BDR91A/T  # TODO: 0008/0009/3150
     """For a small electric load controlled by a relay (never calls for heat)."""
 
-    __zon_class__ = ZONE_CLASS.ELE
+    _TYPE = ZONE_TYPE.ELE
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't super()
+        # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
         if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.ELE}")
+            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.ELE}")
 
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
 
-        # if msg.code == "0008":  # ZON zones are ELE zones that also call for heat
+        # if msg.code == _0008:  # ZON zones are ELE zones that also call for heat
         #     self._set_zone_type("VAL")
-        if msg.code == "3150":
+        if msg.code == _3150:
             raise TypeError("WHAT 1")
-        elif msg.code == "3EF0":
+        elif msg.code == _3EF0:
             raise TypeError("WHAT 2")
 
     @property
@@ -832,88 +793,90 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
     Note that HM80s are listen-only devices.
     """
 
-    __zon_class__ = ZONE_CLASS.MIX
+    _TYPE = ZONE_TYPE.MIX
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't super()
+        # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
+
         if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.MIX}")
+            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.MIX}")
 
         if discover_flag & DISCOVER_PARAMS:
             self._gwy.send_cmd(Command.get_mix_valve_params(self._ctl.id, self.idx))
 
-    def _handle_msg(self, msg) -> bool:
-        super()._handle_msg(msg)
-
-        if msg.code == "1030" and msg.verb == I_:
-            self._mix_config = msg
-
     @property
-    def mix_config(self) -> dict:
-        return self._msg_payload(self._mix_config)
+    def mix_config(self) -> dict:  # 1030
+        return self._msg_value(_1030)
 
     @property
     def params(self) -> dict:
-        return {**super().status, "mix_config": self.mix_config}
+        return {
+            **super().status,
+            "mix_config": self.mix_config,
+        }
 
 
 class RadZone(Zone):  # HR92/HR80
     """For radiators controlled by HR92s or HR80s (will also call for heat)."""
 
-    __zon_class__ = ZONE_CLASS.RAD
+    _TYPE = ZONE_TYPE.RAD
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't super()
+        # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
+
         if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.RAD}")
+            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.RAD}")
 
 
 class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
     """For underfloor heating controlled by an HCE80/HCC80 (will also call for heat)."""
 
-    __zon_class__ = ZONE_CLASS.UFH
+    _TYPE = ZONE_TYPE.UFH
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't super()
+        # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
+
         if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.UFH}")
-
-    def _handle_msg(self, msg) -> bool:
-        super()._handle_msg(msg)
-
-        if msg.code == "22C9" and msg.verb == I_:
-            self._ufh_setpoint = msg
+            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.UFH}")
 
     @property
     def ufh_setpoint(self) -> Optional[float]:  # 22C9
-        return self._msg_payload(self._ufh_setpoint, ATTR_SETPOINT)
+        return self._msg_value(_22C9, key=ATTR_SETPOINT)
 
     @property
     def status(self) -> dict:
-        return {**super().status, "ufh_setpoint": self.ufh_setpoint}
+        return {
+            **super().status,
+            "ufh_setpoint": self.ufh_setpoint,
+        }
 
 
 class ValZone(EleZone):  # BDR91A/T
     """For a motorised valve controlled by a BDR91 (will also call for heat)."""
 
-    __zon_class__ = ZONE_CLASS.VAL
+    _TYPE = ZONE_TYPE.VAL
 
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
+    @discover_decorator
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # NOTE: we create, then promote, so shouldn't super()
+        # NOTE: we create, then promote, so shouldn't (can't) super()
         # super()._discover(discover_flag=discover_flag)
+
         if False and discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.VAL}")
+            self._send_cmd(_000C, payload=f"{self.idx}{_000C_DEVICE.VAL}")
 
     @property
     def heat_demand(self) -> Optional[float]:  # 0008 (NOTE: not 3150)
@@ -921,12 +884,12 @@ class ValZone(EleZone):  # BDR91A/T
         return self.relay_demand
 
 
-CLASS_ATTR = "__zon_class__"
-ZONE_BY_CLASS_ID = {
-    getattr(c[1], CLASS_ATTR): c[1]
+_TYPE = "_TYPE"
+ZONE_BY_TYPE = {
+    getattr(c[1], _TYPE): c[1]
     for c in getmembers(
         modules[__name__],
-        lambda m: isclass(m) and m.__module__ == __name__ and hasattr(m, CLASS_ATTR),
+        lambda m: isclass(m) and m.__module__ == __name__ and hasattr(m, _TYPE),
     )
 }  # e.g. "RAD": RadZone
 
@@ -935,13 +898,19 @@ def create_zone(evo, zone_idx, profile=None, **kwargs) -> Zone:
     """Create a zone, and optionally perform discovery & start polling."""
 
     if profile is None:
-        profile = ZONE_CLASS.DHW if zone_idx == "HW" else None
+        profile = ZONE_TYPE.DHW if zone_idx == "HW" else None
 
-    zone = ZONE_BY_CLASS_ID.get(profile, Zone)(evo, zone_idx, **kwargs)
+    zone = ZONE_BY_TYPE.get(profile, Zone)(evo, zone_idx, **kwargs)
 
-    if not evo._gwy.config.disable_discovery:
-        schedule_task(zone._discover, discover_flag=DISCOVER_SCHEMA, delay=1)
-        schedule_task(zone._discover, discover_flag=DISCOVER_PARAMS, delay=4)
-        schedule_task(zone._discover, discover_flag=DISCOVER_STATUS, delay=7)
+    # if True or not evo._gwy.config.disable_discovery:
+    evo._gwy._add_task(
+        zone._discover, discover_flag=DISCOVER_SCHEMA, delay=2, period=86400
+    )
+    evo._gwy._add_task(
+        zone._discover, discover_flag=DISCOVER_PARAMS, delay=5, period=21600
+    )
+    evo._gwy._add_task(
+        zone._discover, discover_flag=DISCOVER_STATUS, delay=8, period=900
+    )
 
     return zone
